/*****************************************************************************
 *   Copyright (C) 2009 Peter "woggle" Mack, mac@denich.net                  *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *****************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "lcd.h"
#include "timer.h"	// Keys
#include "servo.h"

#define SERVO_CORRECT 3.125

//*****************************************************************************
//
ISR(TIMER1_COMPA_vect)
{
	PORTD |= (1 << PD4);
	PORTD |= (1 << PD5);
}

//*****************************************************************************
//
ISR(TIMER1_COMPB_vect)
{
	PORTD &= ~(1 << PD4);
	PORTD &= ~(1 << PD5);
}

//*****************************************************************************
//
void servo_test (void)
{
	uint8_t chg = 0;
	uint8_t Z = 150;  // 1,5mS

	TIMSK1 = (1 << OCIE1A) | (1 << OCIE1B);

	lcd_cls ();

	lcd_printp (PSTR(" Servo Tester        "), 2);
	lcd_printpns_at (0, 7, PSTR(" \x1c     \x1d    Ende  \x1d\x7c\x1c"), 0);
//	lcd_printpns_at (0, 2, PSTR("15 10  5  0  5  10 15"), 0);
	lcd_printpns_at (7, 5, PSTR("%"), 0);
	lcd_printpns_at (16, 5, PSTR("mS"), 0);

	lcd_rect(3, 23, 120, 8, 1);  // +-150% Rahmen
	lcd_line(23,23,23,31,1);  // -100%
	lcd_line(43,23,43,31,1);  // -50%
	lcd_frect(61, 23, 3, 8, 1);  // 0%
	lcd_line(83,23,83,31,1);  // +50%
	lcd_line(103,23,103,31,1);  // +100%

	write_ndigit_number_u (4, 5, 0, 3, 0);  // Pulse width in %
	write_ndigit_number_u_100th(12, 5, 150, 3, 0);  // Pulse width in ms

	do
	{
		if ((get_key_press (1 << KEY_PLUS) || get_key_rpt (1 << KEY_PLUS)) && (Z < 225))
		{
			lcd_frect (4, 24, 118, 6, 0);  // Balken l�schen
			Z++;
			if (Z >= 225)
			{
				Z = 225;
				beeptime = 200;
				BeepMuster = 0x0080;
			}
			chg++;
		}
		else if ((get_key_press (1 << KEY_MINUS) || get_key_rpt (1 << KEY_MINUS)) && (Z > 75))
		{
			lcd_frect (4, 24, 118, 6, 0);  // Balken l�schen
			Z--;
			if (Z <= 75)
			{
				Z = 75;
				beeptime = 200;
				BeepMuster = 0x0080;
			}
			chg++;
		}
		else if (get_key_press (1 << KEY_ENTER))
		{
			lcd_frect (4, 24, 118, 6, 0);  // Balken l�schen
			lcd_frect(61, 23, 3, 8, 1);  // 0%
			Z = 150;
			chg++;
		}

		if (chg)
		{
			chg = 0;

			if (Z >= 150)
			{
				lcd_frect (63, 24, ((Z - 150) * 0.8), 6, 1);
				write_ndigit_number_u (4, 5, ((Z - 150) * 2), 3, 0);  // Pulse width in %
				lcd_frect(62, 23, 2, 8, 1);  // 0%
			}
			else
			{
				lcd_frect (63 - ((150 - Z) * 0.8), 24, ((150 - Z) * 0.8), 6, 1);
				write_ndigit_number_u (4, 5, ((150 - Z) * 2), 3, 0);  // Pulse width in %
				lcd_frect(61, 23, 2, 8, 1);  // 0%
			}
			write_ndigit_number_u_100th(12, 5, Z, 3, 0);  // Pulse width in ms

			lcd_line(23,23,23,31,1);  // -100%
			lcd_line(43,23,43,31,1);  // -50%
			lcd_line(83,23,83,31,1);  // +50%
			lcd_line(103,23,103,31,1);  // +100%

			OCR1A = Z * SERVO_CORRECT;  // Servo rechts drehend

			if (Z >= 150)				// Servo links drehend
				OCR1B = (150 - (Z - 150)) * SERVO_CORRECT;
			else
				OCR1B = (150 + (150 - Z)) * SERVO_CORRECT;

		}
	}
	while (!get_key_press (1 << KEY_ESC));
	get_key_press(KEY_ALL);

	OCR1A = 150 * SERVO_CORRECT;  // SERVO_CENTER
	OCR1B = 150 * SERVO_CORRECT;  // SERVO_CENTER
	TIMSK1 = 0;
}
